module Mutations
  class DossierPasserEnInstruction < Mutations::BaseMutation
    include DossierHelper

    description "Passer le dossier en instruction."

    argument :dossier_id, ID, "Dossier ID", required: true, loads: Types::DossierType
    argument :instructeur_id, ID, "Instructeur qui prend la décision sur le dossier.", required: true, loads: Types::ProfileType

    field :dossier, Types::DossierType, null: true
    field :errors, [Types::ValidationErrorType], null: true

    def resolve(dossier:, instructeur:)
      dossier.passer_en_instruction!(instructeur)

      { dossier: dossier }
    end

    def authorized?(dossier:, instructeur:)
      if !dossier.en_construction?
        return false, { errors: ["Le dossier est déjà #{dossier_display_state(dossier, lower: true)}"] }
      end
      dossier_authorized_for?(dossier, instructeur)
    end
  end
end
